package de.hawhamburg.reachability.sensor;

import java.util.HashMap;
import java.util.Map;

import de.hawhamburg.reachability.reasoner.enumeration.ReasonerDimension;

/**
 * The common internal representation of sensor data.<br><br>
 * 
 * This class provides a common representation of all sensor data for the further internal use.
 * 
 * @author  {@value de.hawhamburg.reachability.ProjectInfo#REACHABILITY_AGENT_AUTHOR} 
 * @version {@value de.hawhamburg.reachability.ProjectInfo#REACHABILITY_AGENT_VERSION}
 */
public class SensorData {
    
    /**
     * The name of the sensor, which provides this data object.
     */
    private String name                     = null;
    
    /**
     * The {@link ReasonerDimension} for which this data object was measured. 
     */
    private ReasonerDimension dimension     = null;
    
    /**
     * The list of values measured by the sensor.
     */
    private Map<Integer, Double> values     = new HashMap<Integer, Double>();
    
    /**
     * The time when this data was created.
     */
    private final long creationTime;
    
    /**
     * Creates a new sensor data object.
     * 
     * @param sensorName    The name of the sensor, which provides this data object.
     * @param dataDimension The {@link ReasonerDimension} for which this data object was measured.
     * @param sensorValue   The value measured by the sensor.
     */
    public SensorData(final String sensorName,
                      final ReasonerDimension dataDimension,
                      final double sensorValue) {
        this.name         = sensorName;
        this.dimension    = dataDimension;
        this.creationTime = System.currentTimeMillis();
        this.values.put(0, sensorValue);
    }
    
    /**
     * Creates a new sensor data object.
     * 
     * @param sensorName    The name of the sensor, which provides this data object.
     * @param dataDimension The {@link ReasonerDimension} for which this data object was measured.
     * @param sensorValues  The value measured by the sensor.
     */
    public SensorData(final String sensorName,
                      final ReasonerDimension dataDimension,
                      final double[] sensorValues) {
        this.name         = sensorName;
        this.dimension    = dataDimension;
        this.creationTime = System.currentTimeMillis();
        int i = 0;
        for (double d : sensorValues) {
            this.values.put(i++, d);
        }
    }
    
    /**
     * Creates a new sensor data object.
     * 
     * @param sensorName    The name of the sensor, which provides this data object.
     * @param dataDimension The {@link ReasonerDimension} for which this data object was measured.
     * @param sensorValues  The value measured by the sensor.
     */
    public SensorData(final String sensorName,
                      final ReasonerDimension dataDimension,
                      final Map<Integer, Double> sensorValues) {
        this.name           = sensorName;
        this.dimension      = dataDimension;
        this.values         = sensorValues;
        this.creationTime   = System.currentTimeMillis();
    }
    
    /**
     * Returns the name of the sensor, which measured this value.
     * 
     * @return              The name of the sensor, which measured this value, as string.
     */
    public final String getName() {
        return this.name;
    }
    
    /**
     * Returns the data dimension as {@link ReasonerDimension}.
     * 
     * @return              The {@link ReasonerDimension} of this data object.
     */
    public final ReasonerDimension getDimension() {
        return this.dimension;
    }
    
    /**
     * Returns the first value of the value scalar for downwards compatibility.
     *  
     * @return              The first value of the value scalar as double.
     */
    public final double getValue() {
        return getValue(0);
    }
    
    /**
     * Returns the value scalar as Map&lt;Integer, Double&gt;.
     * 
     * @return              A map with the scalars dimension as key an the corresponding value as
     *                      value.
     */
    public final Map<Integer, Double> getValues() {
        return values;
    }
    
    /**
     * Returns the n-th dimension of the value scalar measured by the sensor.
     * 
     * @param index         The dimension as integer.
     * @return              The value of the n-th dimension of the value scalar.
     */
    public final double getValue(final int index) {
        if (values.containsKey(index)) {
            return values.get(index);
        }
        return 0.0;
    }
    
    /**
     * Returns the creation time of this data as UNIX timestamp.
     * 
     * @return              The time of creation as UNIX timestamp.
     */
    protected final long getCreationTime() {
        return this.creationTime;
    }
    
    /**
     * @see java.lang.Object#toString()
     * 
     * @return              The {@link String} representation of this object.
     */
    public final String toString() {
        return this.name + " [" + this.dimension + "] = " + String.valueOf(this.values);
    }
}
