package de.hawhamburg.reachability.util;

import java.util.HashMap;
import java.util.Map;

/**
 * Utility functions related to command line parameters.<br><br>
 * 
 * This utility class provides some fancy methods for easy handling String parameters supplied on system call. 
 * 
 * @author  {@value de.hawhamburg.reachability.ProjectInfo#REACHABILITY_AGENT_AUTHOR} 
 * @version {@value de.hawhamburg.reachability.ProjectInfo#REACHABILITY_AGENT_VERSION}
 */
public final class ParameterUtil {
    
    /**
     * The parameters known to this utility class.
     */
    private Map<String, String> parms;
    
    /**
     * Initializes the known parameters.
     * 
     * @param params    The known parameters.
     */
    private ParameterUtil(final Map<String, String> params) {
        this.parms = params;
    }
    
    /**
     * Creates a {@link ParameterUtil} object of certain attributes from the system supplied argument string array by 
     * using {@link #fetchParms(String[], String[])}.
     * 
     * @param args      The system supplied argument string array.
     * @param subset    The requested subset of arguments.
     * @return          An {@link ParameterUtil} object containing the requested subset of arguments if existing.
     */
    public static ParameterUtil fromArgs(final String[] args,
                                         final String[] subset) {
        return new ParameterUtil(fetchParms(args, subset));
    }
    
    /**
     * Fetches a map of arguments from a system supplied argument string array which contains only the ones supplied in 
     * the subset parameter.
     * 
     * @param args      The system supplied argument string array.
     * @param subset    The requested subset of arguments.
     * @return          A {@link Map} of {@link String} to {@link String} containing the requested system arguments.
     */
    private static Map<String, String> fetchParms(final String[] args,
                                                  final String[] subset) {
        Map<String, String> parms = new HashMap<String, String>();
        for (String x : args) {
            for (String parm : subset) {
                if (x.length() > parm.length() - 1 && x.substring(0, parm.length()).equals(parm)) {
                    if (x.length() > parm.length()) {
                        parms.put(parm, x.substring(parm.length() + 1));
                    } else {
                        parms.put(parm, null);
                    }
                }
            }
        }
        return parms;
    }

    /**
     * Returns the {@link String} value of a supplied key if known to the class or <code>null</code> instead by using 
     * {@link #fetchString(String, String)}.
     * 
     * @param key       The name of the requested argument as {@link String}.
     * @return          The value of the key if known or <code>null</code> instead.
     */
    public String fetchString(final String key) {
        return fetchString(key, null);
    }
    
    /**
     * Returns the {@link String} value of a supplied key if known to the class or the supplied default value instead.
     * 
     * @param key       The name of the requested argument as {@link String}.
     * @param defVal    The default value of the argument as {@link String}, which will be returned in case the key is 
     *                  unknown or undefined.
     * @return          The value of the key if known or the supplied default value instead.
     */
    public String fetchString(final String key,
                              final String defVal) {
        if (this.isset(key)) {
            return parms.get(key);
        }
        return defVal;
    }
    
    /**
     * Returns the <code>int</code> value of a supplied key if known to the class or <code>0</code> instead by using 
     * {@link #fetchInt(String, int)}.
     * 
     * @param key       The name of the requested argument as {@link String}.
     * @return          The value of the key if known or <code>0</code> instead.
     */
    public int fetchInt(final String key) {
        return fetchInt(key, 0);
    }
    
    /**
     * Returns the <code>int</code> value of a supplied key if known to the class or the supplied default value instead.
     * 
     * @param key       The name of the requested argument as {@link String}.
     * @param defVal    The default value of the argument as <code>int</code>, which will be returned in case the key is
     *                  unknown or undefined.
     * @return          The value of the key if known or the supplied default value instead.
     */
    public int fetchInt(final String key,
                        final int defVal) {
        if (this.isset(key)) {
            return Integer.parseInt(parms.get(key));
        }
        return defVal;
    }
    
    /**
     * Returns the <code>boolean</code> value of a supplied key if known to the class or <code>false</code> instead by 
     * using {@link #fetchBool(String, boolean)}.
     * 
     * @param key       The name of the requested argument as {@link String}.
     * @return          The value of the key if known or <code>false</code> instead.
     */
    public boolean fetchBool(final String key) {
        return fetchBool(key, false);
    }
    
    /**
     * Returns the <code>boolean</code> value of a supplied key if known to the class or the supplied default value 
     * instead.
     * 
     * @param key       The name of the requested argument as {@link String}.
     * @param defVal    The default value of the argument as <code>boolean</code>, which will be returned in case the 
     *                  key is unknown or undefined.
     * @return          The value of the key if known or the supplied default value instead.
     */
    public boolean fetchBool(final String key,
                             final boolean defVal) {
        if (this.isset(key)) {
            return Boolean.parseBoolean(parms.get(key));
        }
        return defVal;
    }
    
    /**
     * Checks whether a certain key as {@link String} is known as parameter to the current class instance or not.
     * 
     * @param key       The name of the requested parameter as {@link String}. 
     * @return          <code>true</code> if the parameter is known, otherwise <code>false</code>.
     */
    public boolean isset(final String key) {
        return parms.containsKey(key);
    }
    
    /**
     * Checks if at least one parameter of the supplied {@link String} array is known to this class instance. 
     * 
     * @param keys      A {@link String} array of parameters.
     * @return          <code>true</code> if at least one parameter of the supplied {@link String} array is known to 
     *                  this class instance, otherwise <code>false</code>.
     */
    public boolean someset(final String[] keys) {
        for (String key : keys) {
            if (this.isset(key)) {
                return true;
            }
        }
        return false;
    }
}
